#!/bin/sh
# External commands execution in MIDAS
# Notes: 1. MIDAS version >= 93NOV
#        2. Connection with 94NOV may hang
#           (problems with USR1 signal catching in prepa?)
#           use SET/BACK, CLEAR/BACK commands in this case...
# Usage:
#       cat Midas_commands_list | xmid [-s Mid_sess] [-t Timeout]
# where:
#       Mid_sess - MIDAS session id to comunicate with (default "00")
#       Timeout  - #sec. to wait or "wait" or "nowait" (default "15")
# E.g:
#       xmid -t wait < list
#       echo "read/key mode" | xmid -s 01 -t nowait
#

SIGUSR1=10
SIGUSR2=12

MyUnit=sh
ToUnit=00
if [ "x$1" = "x-s" ] ; then
  shift
  if [ -n "$1" ] ; then
    ToUnit=$1
    shift
  fi
fi
Timeout=15
if [ "x$1" = "x-t" ] ; then
  shift
  if [ -n "$1" ] ; then
    Timeout=$1
    shift
  fi
fi
MidDir=$HOME/midwork
if [ ! -d "$MidDir" ] ; then
   echo "$MidDir - directory absent !"
   exit
fi
MyPID=$$
Name=$MidDir/MIDAS$ToUnit.PID
if [ ! -r "$Name" ] ; then
   echo "Can't find MIDAS session $ToUnit."
   exit
fi
ToPID=`cat $Name`
if kill -0 $ToPID 2>/dev/null ; then
   echo
else
   echo "Can't connect to MIDAS session $ToUnit"
   exit
fi

Sbox=$MidDir/FORGR$ToUnit.SBOX
Rbox=$MidDir/FORGR$ToUnit$MyUnit.RBOX
while read MidCom
do
   echo "$MyUnit $MidCom" >$Sbox
   echo "$MyPID" >>$Sbox

   rm -f $Rbox
   trap "cat $Rbox" $SIGUSR2
   if kill -$SIGUSR1 $ToPID 2>/dev/null ; then
      sleep 0
   else
      echo "Can't connect to MIDAS session $ToUnit"
      exit
   fi
   if [ ! "$Timeout" = "nowait" ] ; then
     Tout=$Timeout
     [ "$Tout" = "wait" ] && Tout=32767
     sleep 0
     while [ -r "$Sbox" -o ! -r "$Rbox" -a "$Tout" -gt 0 ]
     do
       sleep 1
       Tout=`expr $Tout - 1`
     done
     if [ ! -r "$Rbox" ] ; then
	echo "Timeout when access MIDAS session $ToUnit ..."
	exit
     fi
   fi
done
